GECKODRIVER_DOWNLOAD_URL = 'https://github.com/mozilla/geckodriver/releases/download/v{}/geckodriver-v{}-{}'
GITHUB_GECKODRIVER_TAGS_URL = 'https://github.com/mozilla/geckodriver/tags'
GITHUB_GECKODRIVER_VERSION_URL = 'https://github.com/mozilla/geckodriver/releases'
GITHUB_GECKODRIVER_LATEST_VERSION_ANCHOR = '.label-latest > div:nth-child(2) > div:nth-child(1) > div:nth-child(1) > ' \
                                           'div:nth-child(1) > a:nth-child(1) '

ZIP_TYPE = '.zip'
TAR_GZ_TYPE = '.tar.gz'

GECKODRIVER = 'geckodriver'
WEBDRIVER = 'webdriver'
